'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

/**
 * 图像缩放 image is Image or canvas
* @param  {Image | Canvas} image  is Image or canvas
* @param  {number} 			scale  缩放比例
* @return {Canvas}        新的画布
 */
exports.default = function (image) {
    var scale = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0.5;

    var _canvas = document.createElement('canvas'),
        _context = _canvas.getContext('2d');
    var width = image.width,
        height = image.height,
        newWidth = width * scale,
        newHeight = height * scale;

    _canvas.width = newWidth;
    _canvas.height = newHeight;

    if (width > newWidth) {
        var oc = document.createElement('canvas'),
            octx = oc.getContext('2d');
        var steps = Math.ceil(Math.log(scale) / Math.log(2));

        oc.width = width;
        oc.height = height;

        octx.drawImage(image, 0, 0, oc.width, oc.height);

        var _pow = void 0,
            _w = void 0,
            _h = void 0,
            d_w = oc.width,
            d_h = oc.height;
        for (var s = 1; s < steps; s++) {
            _pow = Math.pow(2, s);
            _w = width / _pow;
            _h = height / _pow;
            octx.drawImage(oc, 0, 0, d_w, d_h, 0, 0, _w, _h);
            d_w = _w;
            d_h = _h;
        }
        _context.drawImage(oc, 0, 0, d_w, d_h, 0, 0, newWidth, newHeight);
    } else {
        _context.drawImage(image, 0, 0, newWidth, newHeight);
    }

    return _canvas;
};