'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _Point = require('../math/Point2');

var _Point2 = _interopRequireDefault(_Point);

var _LineCurve = require('./LineCurve');

var _LineCurve2 = _interopRequireDefault(_LineCurve);

var _QuadraticBezierCurve = require('./QuadraticBezierCurve');

var _QuadraticBezierCurve2 = _interopRequireDefault(_QuadraticBezierCurve);

var _CubicBezierCurve = require('./CubicBezierCurve');

var _CubicBezierCurve2 = _interopRequireDefault(_CubicBezierCurve);

var _bezierJs = require('bezier-js');

var _bezierJs2 = _interopRequireDefault(_bezierJs);

var _paperCore = require('paper/dist/paper-core.js');

var _paperCore2 = _interopRequireDefault(_paperCore);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

_paperCore2.default.project = new _paperCore2.default.Project();

var Path = function () {
	function Path(points) {
		_classCallCheck(this, Path);

		this.curveGroups = [];

		this.tempPoint = new _Point2.default();

		if (points) {
			this.fromPoints(points);
		}
	}

	_createClass(Path, [{
		key: 'copy',
		value: function copy(p) {
			this.clear();
			this.curves = p.curveGroups.map(function (g) {
				return g.map(function (c) {
					return c.clone();
				});
			});

			return this;
		}
	}, {
		key: 'clone',
		value: function clone() {
			return new Path().copy(this);
		}
	}, {
		key: 'fromPoints',
		value: function fromPoints(points) {

			this.moveTo(points[0].x, points[0].y);

			for (var i = 1, l = points.length; i < l; i++) {

				this.lineTo(points[i].x, points[i].y);
			}
		}
	}, {
		key: 'moveTo',
		value: function moveTo(x, y) {

			this.tempPoint.set(x, y);

			this.curveGroups.push([]);

			return null;
		}
	}, {
		key: 'lineTo',
		value: function lineTo(x, y) {

			var curve = new _LineCurve2.default(new _Point2.default().copy(this.tempPoint), new _Point2.default(x, y));

			this.tempPoint.set(x, y);

			this.curveGroups[this.curveGroups.length - 1].push(curve);

			return curve;
		}
	}, {
		key: 'quadraticCurveTo',
		value: function quadraticCurveTo(aCPx, aCPy, aX, aY) {

			var curve = new _QuadraticBezierCurve2.default(new _Point2.default().copy(this.tempPoint), new _Point2.default(aCPx, aCPy), new _Point2.default(aX, aY));

			this.tempPoint.set(aX, aY);

			this.curveGroups[this.curveGroups.length - 1].push(curve);

			return curve;
		}
	}, {
		key: 'bezierCurveTo',
		value: function bezierCurveTo(aCP1x, aCP1y, aCP2x, aCP2y, aX, aY) {

			var curve = new _CubicBezierCurve2.default(new _Point2.default().copy(this.tempPoint), new _Point2.default(aCP1x, aCP1y), new _Point2.default(aCP2x, aCP2y), new _Point2.default(aX, aY));

			this.tempPoint.set(aX, aY);

			this.curveGroups[this.curveGroups.length - 1].push(curve);

			return curve;
		}

		// getCurveLengths() {

		// 	// We use cache values if curves and cache array are same length

		// 	if ( this.cacheLengths && this.cacheLengths.length === this.curves.length ) {

		// 		return this.cacheLengths;

		// 	}

		// 	// Get length of sub-curve
		// 	// Push sums into cached array

		// 	let lengths = [], sums = 0;

		// 	for ( let i = 0, l = this.curves.length; i < l; i ++ ) {

		// 		sums += this.curves[ i ].getLength();
		// 		lengths.push( sums );

		// 	}

		// 	this.cacheLengths = lengths;

		// 	return lengths;

		// }

		// getLength() {
		// 	let lens = this.getCurveLengths();
		// 	return lens[ lens.length - 1 ];
		// }

		// getPoint( t ) {

		// 	let d = t * this.getLength();
		// 	let curveLengths = this.getCurveLengths();
		// 	let i = 0;

		// 	// To think about boundaries points.

		// 	while ( i < curveLengths.length ) {

		// 		if ( curveLengths[ i ] >= d ) {

		// 			let diff = curveLengths[ i ] - d;
		// 			let curve = this.curves[ i ];

		// 			let u = 1 - diff / curve.getLength();

		// 			return curve.getPointAt( u );

		// 		}

		// 		i ++;

		// 	}

		// 	return null;

		// 	// loop where sum != 0, sum > d , sum+1 <d

		// }


	}, {
		key: 'getOffset',
		value: function getOffset(distance) {
			var resultPath = new Path();

			for (var g = 0, lg = this.curveGroups.length; g < lg; g++) {
				var group = this.curveGroups[g];
				for (var i = 0, li = group.length; i < li; i++) {
					var curve = group[i];
					var bezier = void 0;
					switch (curve.type) {
						case 'LineCurve':
							bezier = new _bezierJs2.default(curve.v0.toObject(), curve.v1.toObject());
							break;
						case 'QuadraticBezierCurve':
							bezier = new _bezierJs2.default(curve.v0.toObject(), curve.v1.toObject(), curve.v2.toObject());
							break;
						case 'CubicBezierCurve':
							bezier = new _bezierJs2.default(curve.v0.toObject(), curve.v1.toObject(), curve.v2.toObject(), curve.v3.toObject());
							break;
					}
					if (bezier) {
						var beziers = bezier.offset(distance);

						// 删除存在点是NaN的曲线 －－－－－－－－－－－－start－－－－－－－－－－－－
						var mBeziers = [];
						var _iteratorNormalCompletion = true;
						var _didIteratorError = false;
						var _iteratorError = undefined;

						try {
							for (var _iterator = beziers[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
								var b = _step.value;

								var nan = false;

								var _iteratorNormalCompletion2 = true;
								var _didIteratorError2 = false;
								var _iteratorError2 = undefined;

								try {
									for (var _iterator2 = b.points[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
										var p = _step2.value;

										if (isNaN(p.x) || isNaN(p.y)) {
											nan = true;
											break;
										}
									}
								} catch (err) {
									_didIteratorError2 = true;
									_iteratorError2 = err;
								} finally {
									try {
										if (!_iteratorNormalCompletion2 && _iterator2.return) {
											_iterator2.return();
										}
									} finally {
										if (_didIteratorError2) {
											throw _iteratorError2;
										}
									}
								}

								if (!nan) {
									mBeziers.push(b);
								}
							}
							// 删除存在点是NaN的曲线 －－－－－－－－－－－－end－－－－－－－－－－－－
						} catch (err) {
							_didIteratorError = true;
							_iteratorError = err;
						} finally {
							try {
								if (!_iteratorNormalCompletion && _iterator.return) {
									_iterator.return();
								}
							} finally {
								if (_didIteratorError) {
									throw _iteratorError;
								}
							}
						}

						for (var j = 0, lj = mBeziers.length; j < lj; j++) {
							var eachBezier = mBeziers[j];

							var points = eachBezier.points;

							if (!resultPath.curveGroups[g]) {

								resultPath.moveTo(points[0].x, points[0].y);
							}
							var pl = points.length;
							switch (pl) {
								case 2:
									resultPath.lineTo(points[1].x, points[1].y);
									break;
								case 3:
									resultPath.quadraticCurveTo(points[1].x, points[1].y, points[2].x, points[2].y);
									break;
								case 4:
									resultPath.bezierCurveTo(points[1].x, points[1].y, points[2].x, points[2].y, points[3].x, points[3].y);
									break;
							}
						}
					}
				}
			}
			return resultPath;
		}
	}, {
		key: 'getOutline',
		value: function getOutline(distance) {
			var resultPath = new Path();

			var half = distance * 0.5;
			var tempPath1 = this.getOffset(half);
			var tempPath2 = this.getOffset(-half);

			for (var g = 0, lg = tempPath1.curveGroups.length; g < lg; g++) {
				var group1 = tempPath1.curveGroups[g];
				var group2 = tempPath2.curveGroups[g];
				for (var i = 0, li = group1.length; i < li; i++) {
					var curve = group1[i];
					if (i === 0) {
						resultPath.moveTo(curve.v0.x, curve.v0.y);
					}
					var type = curve.type;
					switch (type) {
						case 'LineCurve':
							resultPath.lineTo(curve.v1.x, curve.v1.y);
							break;
						case 'QuadraticBezierCurve':
							resultPath.quadraticCurveTo(curve.v1.x, curve.v1.y, curve.v2.x, curve.v2.y);
							break;
						case 'CubicBezierCurve':
							resultPath.bezierCurveTo(curve.v1.x, curve.v1.y, curve.v2.x, curve.v2.y, curve.v3.x, curve.v3.y);
							break;

					}
				}

				for (var lj = group2.length - 1, j = lj; j >= 0; j--) {
					var _curve = group2[j];
					var _type = _curve.type;
					switch (_type) {
						case 'LineCurve':
							if (j === lj) {
								resultPath.lineTo(_curve.v1.x, _curve.v1.y);
							}
							resultPath.lineTo(_curve.v0.x, _curve.v0.y);
							break;
						case 'QuadraticBezierCurve':
							if (j === lj) {
								resultPath.lineTo(_curve.v2.x, _curve.v2.y);
							}
							resultPath.quadraticCurveTo(_curve.v1.x, _curve.v1.y, _curve.v0.x, _curve.v0.y);
							break;
						case 'CubicBezierCurve':
							if (j === lj) {
								resultPath.lineTo(_curve.v3.x, _curve.v3.y);
							}
							resultPath.bezierCurveTo(_curve.v2.x, _curve.v2.y, _curve.v1.x, _curve.v1.y, _curve.v0.x, _curve.v0.y);
							break;

					}
				}

				resultPath.lineTo(group1[0].v0.x, group1[0].v0.y);
			}

			var paperPath = resultPath.toPaperJsPath();
			paperPath.setClosed(true);
			var _iteratorNormalCompletion3 = true;
			var _didIteratorError3 = false;
			var _iteratorError3 = undefined;

			try {
				for (var _iterator3 = this.curveGroups[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
					var group = _step3.value;

					var paperCircleStart = _paperCore2.default.Path.Circle(new _paperCore2.default.Point(group[0].v0.toArray()), half);
					// console.log(paperPath, paperCircleStart);
					paperPath.unite(paperPath);
					// console.log(paperPath);
					// return this.fromPaperJsPath(paperCircleStart);
				}
			} catch (err) {
				_didIteratorError3 = true;
				_iteratorError3 = err;
			} finally {
				try {
					if (!_iteratorNormalCompletion3 && _iterator3.return) {
						_iterator3.return();
					}
				} finally {
					if (_didIteratorError3) {
						throw _iteratorError3;
					}
				}
			}

			return this.fromPaperJsPath(paperPath);
		}
	}, {
		key: 'fromPaperJsPath',
		value: function fromPaperJsPath(path) {
			this.clear();
			var segments = [].concat(_toConsumableArray(path.getSegments()));
			if (path.closed) {
				segments.push(segments[0]);
			}
			this.moveTo(segments[0].getPoint().x, segments[0].getPoint().y);
			for (var i = 1, l = segments.length; i < l; i++) {
				var upSegment = segments[i - 1];
				var segment = segments[i];
				this.bezierCurveTo(upSegment.getHandleOut().x + upSegment.getPoint().x, upSegment.getHandleOut().y + upSegment.getPoint().y, segment.getHandleIn().x + segment.getPoint().x, segment.getHandleIn().y + segment.getPoint().y, segment.getPoint().x, segment.getPoint().y);
			}

			console.log(path.closed);
			return this;
		}
	}, {
		key: 'toPaperJsPath',
		value: function toPaperJsPath(path) {
			path = path || new _paperCore2.default.Path();
			var _iteratorNormalCompletion4 = true;
			var _didIteratorError4 = false;
			var _iteratorError4 = undefined;

			try {
				for (var _iterator4 = this.curveGroups[Symbol.iterator](), _step4; !(_iteratorNormalCompletion4 = (_step4 = _iterator4.next()).done); _iteratorNormalCompletion4 = true) {
					var group = _step4.value;

					for (var i = 0, l = group.length; i < l; i++) {
						var curve = group[i];
						if (i === 0) {
							path.moveTo(new _paperCore2.default.Point(curve.v0.toArray()));
						}

						switch (curve.type) {
							case 'LineCurve':
								path.lineTo(new _paperCore2.default.Point(curve.v1.toArray()));
								break;
							case 'QuadraticBezierCurve':
								path.quadraticCurveTo(new _paperCore2.default.Point(curve.v1.toArray()), new _paperCore2.default.Point(curve.v2.toArray()));
								break;
							case 'CubicBezierCurve':
								path.cubicCurveTo(new _paperCore2.default.Point(curve.v1.toArray()), new _paperCore2.default.Point(curve.v2.toArray()), new _paperCore2.default.Point(curve.v3.toArray()));
								break;
						}
					}
				}
			} catch (err) {
				_didIteratorError4 = true;
				_iteratorError4 = err;
			} finally {
				try {
					if (!_iteratorNormalCompletion4 && _iterator4.return) {
						_iterator4.return();
					}
				} finally {
					if (_didIteratorError4) {
						throw _iteratorError4;
					}
				}
			}

			return path;
		}
	}, {
		key: 'clear',
		value: function clear() {
			this.curveGroups.length = 0;
		}
	}, {
		key: 'fromFabricJsPath',
		value: function fromFabricJsPath(pathData) {
			this.clear();
			var _iteratorNormalCompletion5 = true;
			var _didIteratorError5 = false;
			var _iteratorError5 = undefined;

			try {
				for (var _iterator5 = pathData[Symbol.iterator](), _step5; !(_iteratorNormalCompletion5 = (_step5 = _iterator5.next()).done); _iteratorNormalCompletion5 = true) {
					var item = _step5.value;

					switch (item[0]) {
						case 'M':
							this.moveTo(item[1], item[2]);
							break;
						case 'L':
							this.lineTo(item[1], item[2]);
							break;

						case 'Q':
							this.quadraticCurveTo(item[1], item[2], item[3], item[4]);
							break;
						case 'C':
							this.bezierCurveTo(item[1], item[2], item[3], item[4], item[5], item[6]);
							break;
					}
				}
			} catch (err) {
				_didIteratorError5 = true;
				_iteratorError5 = err;
			} finally {
				try {
					if (!_iteratorNormalCompletion5 && _iterator5.return) {
						_iterator5.return();
					}
				} finally {
					if (_didIteratorError5) {
						throw _iteratorError5;
					}
				}
			}

			return this;
		}
	}, {
		key: 'toFabricJsPath',
		value: function toFabricJsPath(path) {
			var p = path.path;
			p.length = 0;
			var _iteratorNormalCompletion6 = true;
			var _didIteratorError6 = false;
			var _iteratorError6 = undefined;

			try {
				for (var _iterator6 = this.curveGroups[Symbol.iterator](), _step6; !(_iteratorNormalCompletion6 = (_step6 = _iterator6.next()).done); _iteratorNormalCompletion6 = true) {
					var group = _step6.value;

					for (var i = 0, l = group.length; i < l; i++) {
						var curve = group[i];
						if (i === 0) {
							p.push(['M'].concat(_toConsumableArray(curve.v0.toArray())));
						}

						switch (curve.type) {
							case 'LineCurve':
								p.push(['L'].concat(_toConsumableArray(curve.v1.toArray())));
								break;
							case 'QuadraticBezierCurve':
								p.push(['Q'].concat(_toConsumableArray(curve.v1.toArray().concat(curve.v2.toArray()))));
								break;
							case 'CubicBezierCurve':
								p.push(['C'].concat(_toConsumableArray(curve.v1.toArray().concat(curve.v2.toArray(), curve.v3.toArray()))));
								break;
						}
					}
				}
				// console.log('=> ', p);
			} catch (err) {
				_didIteratorError6 = true;
				_iteratorError6 = err;
			} finally {
				try {
					if (!_iteratorNormalCompletion6 && _iterator6.return) {
						_iterator6.return();
					}
				} finally {
					if (_didIteratorError6) {
						throw _iteratorError6;
					}
				}
			}

			return path;
		}
	}]);

	return Path;
}();

exports.default = Path;